#!/usr/bin/env node
/* eslint-disable global-require */
/* eslint-disable no-console */
/* eslint-disable import/no-dynamic-require */
/* eslint-disable @typescript-eslint/no-var-requires */

const path = require('path');
const fs = require('fs');
const yargs = require('yargs');
const c = require('chalk');
const rollup = require('rollup');
const { getBabelOutputPlugin } = require('@rollup/plugin-babel');
const externals = require('rollup-plugin-node-externals');
const typescript = require('rollup-plugin-typescript2');
const resolve = require('@rollup/plugin-node-resolve');
const commonjs = require('@rollup/plugin-commonjs');
const postcss = require('rollup-plugin-postcss');
const renameNodeModules = require('rollup-plugin-rename-node-modules');

const BASEDIR = process.cwd();

console.log(
  c.cyan(`Running ${c.bold('highlight-build')} in ${c.bold(BASEDIR)}`),
);

const options = yargs.usage('Usage: -t <path/to/tsconfig.json>').option('t', {
  alias: 'tsconfig',
  describe: 'Path to tsconfig.json to be used to build the package',
  type: 'string',
}).argv;

const INPUT_FILE = path.resolve(BASEDIR, 'index.ts');
if (!fs.existsSync(INPUT_FILE)) {
  console.error(
    c.red(
      `Entry file ${c.bold(INPUT_FILE)} has not been found in ${c.bold(
        BASEDIR,
      )}`,
    ),
  );
  process.exit(1);
}
console.log(c.gray(`entry file: ${INPUT_FILE}`));

const PKG_FILE = path.resolve(BASEDIR, 'package.json');
if (!fs.existsSync(PKG_FILE)) {
  console.error(
    c.red(`${c.bold('package.json')} file was not found in ${c.bold(BASEDIR)}`),
  );
  process.exit(1);
}
console.log(c.gray(`package.json file: ${PKG_FILE}`));

let tsConfig = {
  cwd: BASEDIR,
  tsconfigOverride: {
    exclude: ['./stories/', './tests/'],
  },
};
if (options.tsconfig) {
  if (!fs.existsSync(path.resolve(BASEDIR, options.tsconfig))) {
    console.error(
      c.red(
        `Cannot file the TS config file ${c.bold(options.tsconfig)} in ${c.bold(
          BASEDIR,
        )}`,
      ),
    );
    process.exit(1);
  }
  console.log(c.gray(`tsconfig: ${path.resolve(BASEDIR, options.tsconfig)}`));
  tsConfig = {
    tsconfig: path.resolve(BASEDIR, options.tsconfig),
  };
}
console.log('No TS config file was specified, will use the default TS config');

async function build() {
  const PKG = require(PKG_FILE);
  console.log(c.yellow('>> ') + c.yellowBright(`Building ${PKG.name}`));

  const inputOptions = {
    input: INPUT_FILE,
    plugins: [
      renameNodeModules(),
      getBabelOutputPlugin({
        plugins: ['@babel/plugin-transform-react-pure-annotations'],
      }),
      externals({
        packagePath: PKG_FILE,
        peerDeps: true,
        deps: false,
        include: /^@highlight-ui/,
      }),
      resolve.default(),
      commonjs(),
      typescript(tsConfig),
      postcss({
        extract: true,
        modules: {
          generateScopedName: `[name]__[hash:base64:8]__v${PKG.version}`,
        },
        use: ['sass'],
      }),
    ],
  };

  const outputOptions = [
    {
      dir: path.resolve(BASEDIR, 'dist/cjs'),
      format: 'cjs',
      sourcemap: true,
    },
    {
      dir: path.resolve(BASEDIR, 'dist/esm'),
      format: 'esm',
      sourcemap: true,
    },
  ];

  const bundle = await rollup.rollup(inputOptions);

  // or write the bundle to disk
  try {
    await Promise.all(
      outputOptions.map((conf) => {
        console.log(
          `${c.blue('>>>')} Writing ${c.blueBright(
            conf.format,
          )} build of ${c.blueBright(PKG.name)} to ${c.blueBright(
            `dist/${conf.format}`,
          )}`,
        );
        return bundle.write(conf);
      }),
    );
  } catch (e) {
    console.error(c.red(String(e)));
  }

  await bundle.close();

  console.log(c.green('>>> ') + c.greenBright(`Done`));
}

build();
